package com.example.controlbrazorobotico.Bluetooth;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;
import android.os.SystemClock;
import android.util.Log;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.UUID;

public class ConectionBT
{
    private int tiempoEspera = 10000000;
    private BluetoothSocket btSocket = null;
    private static final UUID MY_UUID = UUID
            .fromString("00001101-0000-1000-8000-00805F9B34FB");
    private InputStream inStream = null;
    public boolean conected = false;

    public boolean Connect(String address)
    {
        BluetoothAdapter mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
        //Log.d("DEBUG", address);
        BluetoothDevice device = mBluetoothAdapter.getRemoteDevice(address);
        //Log.d("DEBUG", "ConectionBT => Conectando con ... " + device);
        mBluetoothAdapter.cancelDiscovery();
        try
        {
            //Log.d("DEBUG", "ConectionBT => Creando el Socket");
            btSocket = device.createRfcommSocketToServiceRecord(MY_UUID);
            btSocket.connect();
            //Log.d("DEBUG", "ConectionBT => Conexión hecha");
            if(read(tiempoEspera).equals("Y"))
            {
                conected = true;
                return true;
            }
            else
            {
                conected = false;
                return false;
            }
        }
        catch (IOException e)
        {
            try
            {
                btSocket.close();
            }
            catch (IOException e2)
            {
                //Log.d("DEBUG", "ConectionBT => Error al cerrar la conexion");
            }
            //Log.d("DEBUG", "ConectionBT => La conexión no se ha podido establecer");
            return false;
        }
    }

    public String read(int tiempo)
    {
        try
        {
            //Log.d("DEBUG","ConectionBT => Obtenemos el Flujo de entrada");
            inStream = btSocket.getInputStream();
        }
        catch (IOException ignored) {}
        String salida = "";
        try
        {
            long Init = System.nanoTime();
            long Ahora = System.nanoTime();
            byte[] recibido = new byte[255];
            //Log.d("DEBUG","ConectionBT => Leemos");
            inStream.read(recibido);
            while(Ahora-Init < tiempo)
            {
                //Log.d("DEBUG","ConectionBT => Leemos");
                Ahora = System.nanoTime();
                String nuevo = new String(recibido, "US-ASCII");
                if(nuevo.indexOf(0)!=-1)
                {
                    // En el caso de tener el caracter 0 en la ristra, copiamos hasta él
                    salida += nuevo.substring(0,nuevo.indexOf(0));
                    //Log.d("DEBUG","ConectionBT => " + salida);
                    break;
                }
                salida += nuevo;
                //Log.d("DEBUG","ConectionBT => " + salida);
                inStream.read(recibido);
            }
        }
        catch (IOException ignored) {}
        //Log.d("DEBUG","ConectionBT => Devolvemos: " + salida);
        return salida;
    }

    private void write(String data)
    {
        try
        {
            //Log.d("DEBUG","ConectionBT => Enviamos: " + data);
            OutputStream outStream = btSocket.getOutputStream();
            byte[] msgBuffer = (data+"\r").getBytes("US-ASCII");
            outStream.flush();
            outStream.write(msgBuffer);
            //while(!read(tiempoEspera).equals(data+'\r'))
            //    outStream.write(msgBuffer);
            outStream.flush();
        }
        catch (UnsupportedEncodingException ignored) {} catch (IOException ignored) {}
    }

    public boolean sendInstructions(String[] instructions, int n, ArrayList<String> codigosValidos)
    {
        if(n<1)
        {
            return false;
        }
        else
        {
            for(int i=0; i<n; i++)
            {
                if(instructions[i].contains("*"))
                {
                    write(instructions[i].substring(1));
                    String leido = read(tiempoEspera);
                    while(!leido.contains(""+(char)13))
                    {
                        leido += read(tiempoEspera);
                        if(leido.equals(""))
                            write(instructions[i].substring(1));
                    }
                    while(!codigosValidos.contains(leido))
                    {
                        SystemClock.sleep(500);
                        write(instructions[i].substring(1));
                        //Log.d("DEBUG","ConectionBT => Hemos recibido: " + leido + " en lugar de " + 0 +'\r');
                        leido = "";
                        while(!leido.contains(""+(char)13))
                        {
                            leido += read(tiempoEspera);
                            if(leido.equals(""))
                                write(instructions[i].substring(1));
                        }
                    }
                }
                else
                {
                    write(instructions[i]);
                }
                SystemClock.sleep(200);
            }
        }
        return true;
    }

    public void Disconnect()
    {
        write("#");
        try
        {
            btSocket.close();
        }
        catch (IOException ignored) {}
    }

    public ArrayList<String> ObtenInstrucciones(String instruccion, ArrayList<String> codigosValidos)
    {
        ArrayList<String> listaInstrucciones = new ArrayList<String>();
        if (!sendInstructions(instruccion.split(","), instruccion.split(",").length, codigosValidos))
            return listaInstrucciones;
        String leido = "";
        while(!leido.contains(""+(char)26))
        {
            leido += read(tiempoEspera);
        }
        listaInstrucciones.addAll(Arrays.asList(leido.split(""+(char)10)));
        return listaInstrucciones;
    }

    public void EnviaPrograma(ArrayList<String> programa, ArrayList<String> codigosValidos)
    {
        try
        {
            OutputStream outStream = btSocket.getOutputStream();
            for(String linea : programa)
            {

                byte[] msgBuffer = (linea).getBytes("US-ASCII");
                outStream.flush();
                outStream.write(msgBuffer);
                outStream.flush();
            }
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
    }
}
