package com.example.controlbrazorobotico.util;

//import android.util.Log;
import java.io.*;
import java.util.*;


public class Robot {
	
	private class Orden
	{
		public String Instruccion;
		public String Comentario;
		public ArrayList<String[]> Parametros;
		
		public Orden(String Instruct, ArrayList<String[]> param, String comment)
		{
			Parametros = new ArrayList<String[]>();
			Instruccion = Instruct;
            for (String[] aParam : param)
                Parametros.add(aParam);
			Comentario = comment;
		}
	}

	private class RangoMotor
	{
		public int IntInicial;
		public int IntFinal;
		public int AnglInicial;
		public int AnglFinal;
		
		public RangoMotor(int iInit, int iFin, int aInit, int aFin)
		{
			IntInicial = iInit;
			IntFinal = iFin;
			AnglInicial = aInit;
			AnglFinal = aFin;
		}
	}
	
	public String Nombre;
	public int NumBrazos;
	private ArrayList<RangoMotor> RangoBrazos;
	private ArrayList<Orden> Ordenes;
    public ArrayList<String> ComandosBasicos;
    public ArrayList<String> codigoCorrecto;
    public String CabeceraFichero;
	
	public Robot(String fichero)
	{
		try
		{
			InputStream file = new FileInputStream(fichero);
			if (file != null) 
			{
		        InputStreamReader inputreader = new InputStreamReader(file);
			    BufferedReader buffreader = new BufferedReader(inputreader);
			 
			    String line;
			    line = buffreader.readLine();
			    Nombre = line;
			    line = buffreader.readLine();
			    NumBrazos = Integer.parseInt(line);
			    RangoBrazos = new ArrayList<RangoMotor>();
                // Leemos los rangos de movimiento de cada brazo
                for(int i=0; i<NumBrazos; i++)
                {
                    line = buffreader.readLine();
                    String angl = line.substring(line.indexOf(',')+1);
                    if(angl.length()==line.length()+1)
                    {
                        int init = angl.indexOf("..");
                        RangoBrazos.add(new RangoMotor(-1,-1,Integer.parseInt(angl.substring(0, init)),
                                Integer.parseInt(angl.substring(init+2))));
                    }
                    else
                    {
                        if(line.indexOf(',')==-1)
                        {
                            int init = line.indexOf("..");
                            RangoBrazos.add(new RangoMotor(Integer.parseInt(line.substring(0, init)),
                                    Integer.parseInt(line.substring(init+2)), -1,-1));
                        }
                        else
                        {
                            //Log.v("DEBUG", "El brazo distpone tanto de medidas angulares como de valores enteros");
                            int init = line.indexOf("..");
                            int init2 = angl.indexOf("..");
                            RangoBrazos.add(new RangoMotor(Integer.parseInt(line.substring(0, init)),
                                    Integer.parseInt(line.substring(init+2, line.indexOf(','))),
                                    Integer.parseInt(angl.substring(0, init2)),
                                    Integer.parseInt(angl.substring(init2+2))));
                        }
                    }
                }
                Ordenes = new ArrayList<Orden>();
                // Leemos los comandos básicos
                ComandosBasicos = new ArrayList<String>();
                // Conexión
                line = buffreader.readLine();
                ComandosBasicos.add(line);
                // Hard Home
                line = buffreader.readLine();
                ComandosBasicos.add(line);
                // Mover Motor
                line = buffreader.readLine();
                ComandosBasicos.add(line);
                // Definir Posicion Motor
                line = buffreader.readLine();
                ComandosBasicos.add(line);
                // Abrir Pinza
                line = buffreader.readLine();
                ComandosBasicos.add(line);
                // Cerrar Pinza
                line = buffreader.readLine();
                ComandosBasicos.add(line);
                // Recibir Programa
                line = buffreader.readLine();
                ComandosBasicos.add(line);
                // Enviar Programa
                line = buffreader.readLine();
                ComandosBasicos.add(line);
                // Definir Etiqueta
                line = buffreader.readLine();
                ComandosBasicos.add(line);
                // Ir a la Etiqueta
                line = buffreader.readLine();
                ComandosBasicos.add(line);
                // Leer Entrada
                line = buffreader.readLine();
                ComandosBasicos.add(line);
                // Estribir en la Salida
                line = buffreader.readLine();
                ComandosBasicos.add(line);
                // Definir Soft Home
                line = buffreader.readLine();
                ComandosBasicos.add(line);
                // Ir al Soft Home
                line = buffreader.readLine();
                ComandosBasicos.add(line);
                // Parar los Motores
                line = buffreader.readLine();
                ComandosBasicos.add(line);
                // Ejecutar el Programa de la EEPROM
                line = buffreader.readLine();
                ComandosBasicos.add(line);
                // Parar programa de la EEPROM
                line = buffreader.readLine();
                ComandosBasicos.add(line);
                // Leemos los códigos de mensaje correcto
                line = buffreader.readLine();
                codigoCorrecto = new ArrayList<String>();
                if(line.equals("-"))
                    codigoCorrecto.add(line);
                else
                {
                    for(int i=0; i<line.split(";").length; i++)
                    {
                        String palabra = line.split(";")[i];
                        String codigo = "";
                        for(int j=0; j<palabra.split(",").length; j++)
                        {
                            //Log.d("DEBUG", "Codigo: "+codigo);
                            codigo += (char) Integer.parseInt(palabra.split(",")[j]);
                        }
                        //Log.d("DEBUG", "Codigo: "+codigo);
                        codigoCorrecto.add(codigo);
                    }
                }
                // Leemos la cabecera del fichero
                CabeceraFichero = buffreader.readLine();
                // Leemos el conjunto de instrucciones
                line = buffreader.readLine();
                while(line != null)
                {
                    int init = line.indexOf('_');
                    String parametros[] = line.substring(0, init-1).split(",");
                    ArrayList<String[]> listParam = new ArrayList<String[]>();
                    for(int i=1; i<parametros.length; i++)
                    {
                        //Log.v("DEBUG", "	Parametro "+i+ " : '"+parametros[i]+"'");
                        if(parametros[i].contains("IRANGE"))
                        {
                            //Log.v("DEBUG", "		Depende del rango en valores enteros");
                            String[] v = new String[1];
                            v[0] = "IRANGE";
                            listParam.add(v);
                        }
                        else
                        {
                            if(parametros[i].contains("ARANGE"))
                            {
                                //Log.v("DEBUG", "		Depende del rango en valores angulares");
                                String[] v = new String[1];
                                v[0] = "ARANGE";
                                listParam.add(v);
                            }
                            else
                            {
                                int pos = parametros[i].indexOf("..");
                                String x[] = new String[2];
                                x[0] = parametros[i].substring(0, pos);
                                x[1] = parametros[i].substring(pos+2);
                                //Log.v("DEBUG", "		El rango es: "+x[0]+".."+x[1]);
                                listParam.add(x);
                            }
                        }
                    }
                    Ordenes.add(new Orden(parametros[0], listParam, line.substring(init)+1));
                    line = buffreader.readLine();
                }
                buffreader.close();
			}
			file.close();
		}
		catch (IOException e) 
		{
			e.printStackTrace();
		}
	}

	// Funcion que nos da el listado de ordenes disponibles
	public ArrayList<String> ListadoOrdenes()
	{
        ArrayList<String> Listado = new ArrayList<String>();
        for (Orden Ordene : Ordenes)
            Listado.add(Ordene.Instruccion);
		return Listado;	
	}
	
	// Funcion que nos da el numero de parametros de una orden en concreto
	public int NumeroParametros(String Instruccion)
	{
        for (Orden Ordene : Ordenes)
            if (Ordene.Instruccion.equals(Instruccion))
                return Ordene.Parametros.size();
		return -1;
	}
	
	// Funcion que nos da el rango de un parametro de una orden en concreto
	public String[] RangoParametro(String Instruccion, int NParametro)
	{
        for (Orden Ordene : Ordenes)
            if (Ordene.Instruccion.equals(Instruccion) && Ordene.Parametros.size() > NParametro)
                return Ordene.Parametros.get(NParametro);
		return null;
	}

	// Funcion que nos da el comentario de una orden en concreto
	public String ComentarioOrden(String Instruccion)
	{
        for (Orden Ordene : Ordenes)
            if (Ordene.Instruccion.equals(Instruccion))
                return Ordene.Comentario;
		return "ERRORNOEXISTE";
	}
	
	// Funcion que nos da el rango de giro en ticks de un brazo en concreto
	public int[] RangoTickBrazo(int N)
	{
        return new int[]{RangoBrazos.get(N).IntInicial, RangoBrazos.get(N).IntFinal};
	}
	
	// Funcion que nos da el rango de giro en angulos de un brazo en concreto
	public int[] RangoAnguloBrazo(int N)
	{
        return new int[]{RangoBrazos.get(N).AnglInicial, RangoBrazos.get(N).AnglFinal};
	}
}
